#    Adapted from:
#    Copyright (c) 2014 Rowan Nairn. All rights reserved. Licensed under the 2-clause BSD licence
#    https://github.com/r0wb0t/markdown-urlize

import markdown

# Global Vars
URLIZE_RE = '(%s)' % '|'.join([
    r'<(?:f|ht)tps?://[^>]*>',
    r'\b(?:f|ht)tps?://[^)<>\s]+[^.,)<>\s]',
    r'\bwww\.[^)<>\s]+[^.,)<>\s]',
    r'[^(<\s]+\.(?:(?:com|net|org)(?:\.au)?)\b',
    r'doi: *(?:[^ ]+)'
])

class UrlizePattern(markdown.inlinepatterns.Pattern):
    """ Return a link Element given an autolink (`http://example/com`). """
    def handleMatch(self, m):
        url = m.group(2)
        
        if url.startswith('<'):
            url = url[1:-1]
            
        text = url
        
        if not url.split('://')[0] in ('http','https','ftp'):
            if '@' in url and not '/' in url:
                url = 'mailto:' + url
            elif url.startswith('doi:'):
                url = 'https://doi-org.ezproxy.lib.monash.edu.au/' + url[4:].strip()
            else:
                url = 'http://' + url
    
        el = markdown.util.etree.Element("a")
        el.set('href', url)
        el.text = markdown.util.AtomicString(text)
        return el

class UrlizeExtension(markdown.Extension):
    """ Urlize Extension for Python-Markdown. """

    def extendMarkdown(self, md, md_globals):
        """ Replace autolink with UrlizePattern """
        md.inlinePatterns['autolink'] = UrlizePattern(URLIZE_RE, md)

def makeExtension(*args, **kwargs):
    return UrlizeExtension(*args, **kwargs)

if __name__ == "__main__":
    import doctest
    doctest.testmod()
